@@ -39,7 +39,7 @@ def convert(model, file_path=None, input_scaling=None, input_is_image=True):
 
             input_akida = input_scaling[0] * input_keras + input_scaling[1].
 
-        If a :class:`tf.keras.layers.experimental.preprocessing.Rescaling`
+        If a :class:`tf.keras.layers.Rescaling`
         layer is present as first layer of the model, ``input_scaling`` must
         be None: the :class:`Rescaling` parameters will be used to compute the
         input scaling.
@@ -161,7 +161,7 @@ def _get_rescaling_layer_params(model):
             no Rescaling layer is at the beginning of the model.
     """
 
-    Rescaling = tf.keras.layers.experimental.preprocessing.Rescaling
+    Rescaling = tf.keras.layers.Rescaling
     for layer in model.layers[:2]:
         if isinstance(layer, Rescaling):
             return (1 / layer.scale, -layer.offset / layer.scale)
