@@ -20,7 +20,7 @@ Custom tf.keras.constraint.Constraint object to limit weights minimum value.
 
 import tensorflow as tf
 
-from tensorflow.keras.constraints import Constraint
+from keras.constraints import Constraint
 
 
 class MinValueConstraint(Constraint):
