@@ -18,13 +18,11 @@
 
 # Tensorflow imports
 import tensorflow as tf
-from tensorflow.keras import backend as K
-from tensorflow.keras import layers
-from tensorflow.python.keras.layers.ops import core as core_ops  # pylint: disable=no-name-in-module
-from tensorflow.keras.utils import serialize_keras_object
-from tensorflow.python.keras.utils import conv_utils  # pylint: disable=no-name-in-module
+from keras import backend as K
+from keras import layers
+from keras.utils.generic_utils import serialize_keras_object
+from keras.utils import conv_utils  # pylint: disable=no-name-in-module
 from tensorflow.python.ops import nn  # pylint: disable=no-name-in-module
-from tensorflow.python.ops import nn_ops  # pylint: disable=no-name-in-module
 from .quantization_ops import get as get_quantizer
 from .quantization_ops import ceil_through
 
@@ -68,8 +66,8 @@ class QuantizedConv2D(layers.Conv2D):
             regularization applied to the weights.
         bias_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
             regularization applied to the bias.
-        activity_regularizer (str, or a :obj:`tf.keras.regularizer`,
-            optional): regularization applied to the output of the layer.
+        activity_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
+            regularization applied to the output of the layer.
         kernel_constraint (str, or a :obj:`tf.keras.constraint`, optional):
             constraint applied to the weights.
         bias_constraint (str, or a :obj:`tf.keras.constraint`, optional):
@@ -128,27 +126,33 @@ class QuantizedConv2D(layers.Conv2D):
             :obj:`tensorflow.Tensor`: output Tensor.
 
         """
+        input_shape = inputs.shape
+
         outputs = self._convolution_op(inputs,
                                        self.quantizer.quantize(self.kernel))
-
         if self.use_bias:
-            # Handle multiple batch dimensions.
             output_rank = outputs.shape.rank
+            # Handle multiple batch dimensions.
             if output_rank is not None and output_rank > 2 + self.rank:
 
                 def _apply_fn(o):
-                    return nn.bias_add(o,
-                                       self.bias,
-                                       data_format=self._tf_data_format)
-
-                outputs = nn_ops.squeeze_batch_dims(outputs,
-                                                    _apply_fn,
-                                                    inner_rank=self.rank + 1)
+                    return tf.nn.bias_add(o,
+                                          self.bias,
+                                          data_format=self._tf_data_format)
 
+                outputs = conv_utils.squeeze_batch_dims(outputs,
+                                                        _apply_fn,
+                                                        inner_rank=self.rank +
+                                                        1)
             else:
-                outputs = nn.bias_add(outputs,
-                                      self.bias,
-                                      data_format=self._tf_data_format)
+                outputs = tf.nn.bias_add(outputs,
+                                         self.bias,
+                                         data_format=self._tf_data_format)
+
+        if not tf.executing_eagerly():
+            # Infer the static output shape:
+            out_shape = self.compute_output_shape(input_shape)
+            outputs.set_shape(out_shape)
 
         return outputs
 
@@ -182,8 +186,8 @@ class QuantizedDepthwiseConv2D(layers.DepthwiseConv2D):
             regularization applied to the weights.
         bias_regularizer (str, or a :obj:`tf.keras.initializer`, optional):
             regularization applied to the bias.
-        activity_regularizer (str, or a :obj:`tf.keras.regularizer`,
-            optional): regularization applied to the output of the layer.
+        activity_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
+            regularization applied to the output of the layer.
         depthwise_constraint (str, or a :obj:`tf.keras.initializer`, optional):
             constraint applied to the weights.
         bias_constraint (str, or a :obj:`tf.keras.initializer`, optional):
@@ -275,8 +279,8 @@ class QuantizedDense(layers.Dense):
             regularization applied to the weights.
         bias_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
             regularization applied to the bias.
-        activity_regularizer (str, or a :obj:`tf.keras.regularizer`,
-            optional): regularization applied to the output of the layer.
+        activity_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
+            regularization applied to the output of the layer.
         kernel_constraint (str, or a :obj:`tf.keras.constraint`, optional):
             constraint applied to the weights.
         bias_constraint (str, or a :obj:`tf.keras.constraint`, optional):
@@ -324,12 +328,39 @@ class QuantizedDense(layers.Dense):
             :obj:`tensorflow.Tensor`: output Tensor.
 
         """
+        if inputs.dtype.base_dtype != self._compute_dtype_object.base_dtype:
+            inputs = tf.cast(inputs, dtype=self._compute_dtype_object)  # pylint: disable=unexpected-keyword-arg, no-value-for-parameter
+
+        if isinstance(inputs, tf.RaggedTensor):
+            raise TypeError(f"The inputs of a Dense Layer must be a uniform \
+            Tensor. Received {type(inputs)}")
+
         kernel = self.quantizer.quantize(self.kernel)
-        return core_ops.dense(inputs,
-                              kernel,
-                              self.bias,
-                              self.activation,
-                              dtype=self._compute_dtype_object)
+        rank = inputs.shape.rank
+        if rank == 2 or rank is None:
+            # We use embedding_lookup_sparse as a more efficient matmul
+            # operation for large sparse input tensors. The op will result in a
+            # sparse gradient, as opposed to
+            # sparse_ops.sparse_tensor_dense_matmul which results in dense
+            # gradients. This can lead to sigfinicant speedups, see b/171762937.
+            if isinstance(inputs, tf.SparseTensor):
+                raise TypeError(
+                    f"The inputs of a Dense Layer must be a uniform \
+                        Tensor. Received {type(inputs)}")
+            outputs = tf.raw_ops.MatMul(a=inputs, b=kernel)
+            # Broadcast kernel to inputs.
+        else:
+            outputs = tf.tensordot(inputs, kernel, [[rank - 1], [0]])
+            # Reshape the output back to the original ndim of the input.
+            if not tf.executing_eagerly():
+                shape = inputs.shape.as_list()
+                output_shape = shape[:-1] + [kernel.shape[-1]]
+                outputs.set_shape(output_shape)
+
+        if self.use_bias:
+            outputs = tf.nn.bias_add(outputs, self.bias)
+
+        return outputs
 
     def get_config(self):
         config = super().get_config()
@@ -370,8 +401,8 @@ class QuantizedSeparableConv2D(layers.SeparableConv2D):
             regularization applied to the pointwise kernel.
         bias_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
             regularization applied to the bias.
-        activity_regularizer (str, or a :obj:`tf.keras.regularizer`,
-            optional): regularization applied to the output of the layer.
+        activity_regularizer (str, or a :obj:`tf.keras.regularizer`, optional):
+            regularization applied to the output of the layer.
         depthwise_constraint (str, or a :obj:`tf.keras.constraint`, optional):
             constraint applied to the depthwise kernel.
         pointwise_constraint (str, or a :obj:`tf.keras.constraint`, optional):
@@ -540,7 +571,9 @@ class QuantizedActivation(layers.Layer):
         clip_act = tf.clip_by_value(act, 0, self.levels)
         return self.step_height * clip_act
 
-    def call(self, inputs, **kwargs):
+    def call(self, inputs, *args, **kwargs):
+        """Evaluates the quantized activations for the specified input Tensor.
+        """
         return self.quantized_activation(inputs)
 
     @property
