@@ -18,9 +18,9 @@
 
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import
-import tensorflow.keras.backend as K
+import keras.backend as K
 import tensorflow as tf
-from tensorflow.keras.layers import Layer
+from keras.layers import Layer
 
 
 class WeightQuantizer(Layer):
@@ -64,6 +64,11 @@ class WeightQuantizer(Layer):
     """
 
     def __init__(self, bitwidth, **kwargs):
+        if bitwidth <= 1:
+            raise ValueError(
+                f"The quantization bitwidth should be higher than 1.\
+                    Receives bitwidth {bitwidth}")
+
         self.bitwidth_ = int(bitwidth)
         self.kmax_ = (2.**(bitwidth - 1) - 1)
         super().__init__(**kwargs)
@@ -108,7 +113,8 @@ class WeightQuantizer(Layer):
           w (:obj:`tensorflow.Tensor`): the weights Tensor to quantize.
 
         Returns:
-          :obj:`tensorflow.Tensor`: a Tensor containing a single scalar value.
+          :obj:`tensorflow.Tensor`: a Tensor containing a list of scalar values
+                (1 or more).
 
         """
         raise NotImplementedError()
