@@ -21,9 +21,9 @@ can be quantized.
 
 """
 import tensorflow as tf
-from tensorflow.keras.layers import (Conv2D, SeparableConv2D, Dense, MaxPool2D,
-                                     GlobalAveragePooling2D, BatchNormalization)
-from tensorflow.keras.models import (load_model, model_from_json, clone_model)
+from keras.layers import (Conv2D, SeparableConv2D, Dense, MaxPool2D,
+                          GlobalAveragePooling2D, BatchNormalization)
+from keras.models import (load_model, model_from_json, clone_model)
 from .quantization_layers import (QuantizedConv2D, QuantizedSeparableConv2D,
                                   QuantizedDense)
 from .transforms.sequential import compute_BN_folded_weights
@@ -201,7 +201,7 @@ def _prepare_model_to_fold_BN(model, map_neural_layer_to_bn,
 
     # Remove BN layers from model.layers.
     for layer_bn in map_neural_layer_to_bn.values():
-        model._layers.remove(layer_bn)  # pylint: disable=protected-access
+        model._self_tracked_trackables.remove(layer_bn)  # pylint: disable=protected-access
 
     # Update inbound nodes as there is no BN between neural layer and
     # following layer. Inbound nodes are used in "clone_layer" function to
@@ -239,7 +239,7 @@ def load_quantized_model(filepath, custom_objects=None, compile_model=True):
         compile_model (bool): whether to compile the model after loading.
 
     Returns:
-        :obj:`tensorflow.keras.Model`: a Keras model instance.
+        :obj:`keras.Model`: a Keras model instance.
     """
     if custom_objects is None:
         custom_objects = {}
@@ -258,8 +258,8 @@ def load_partial_weights(dest_model, src_model):
     destination layer.
 
     Args:
-        dest_model(:obj:`tensorflow.keras.Model`): the destination Model
-        src_model(:obj:`tensorflow.keras.Model`): the source Model
+        dest_model(:obj:`keras.Model`): the destination Model
+        src_model(:obj:`keras.Model`): the source Model
 
     """
     for src_layer in src_model.layers:
@@ -280,11 +280,11 @@ def create_trainable_quantizer_model(quantized_model):
     such a model to an equivalent model using trainable quantizers.
 
     Args:
-        quantized_model(str, :obj:`tensorflow.keras.Model`): a keras Model or a
+        quantized_model(str, :obj:`keras.Model`): a keras Model or a
         path to a keras Model file
 
     Returns:
-        :obj:`tensorflow.keras.Model`: a Keras Model instance.
+        :obj:`keras.Model`: a Keras Model instance.
     """
     if isinstance(quantized_model, str):
         # Load the model at the specified path
